module Ray
  ( Ray (..),
    rayAt,
    calculateRayNormal,
    buildRay,
  )
where

import Control.Monad.Random
import Linear.V3
import Linear.Vector
import Maths

data Ray f = Ray (V3 f) (V3 f)

-- P(t) = A + tB; A is origin, B is direction
rayAt :: (Floating f) => Ray f -> f -> V3 f
rayAt (Ray origin dir) t = origin + (t *^ dir)

-- get the normal of the point at t
calculateRayNormal :: (Floating f) => Ray f -> f -> V3 f
calculateRayNormal r t = unitVector (rayAt r t ^-^ V3 0.0 0.0 1.0)

buildRay :: (Floating f) => f -> f -> Ray f
buildRay u v = Ray (V3 0.0 0.0 0.0) (V3 v u (-1.0))
